#ifndef MY_CORRESPOND_H
#define MY_CORRESPOND_H

#define MAX_CORRESPOND 8

#ifdef __cplusplus
extern "C"{
#endif

  // 対応点組を表現するための構造体
  // 最大8点組まで表現可能とする
  typedef struct{
    CvPoint2D32f point[MAX_CORRESPOND];
    int num;
  }CvMyCorrespond;

  
  static __inline CvMyCorrespond cvMyCorrespond2(CvPoint2D32f pt1,
						 CvPoint2D32f pt2){
    CvMyCorrespond ret;
    ret.num = 2;
    ret.point[0] = pt1;
    ret.point[1] = pt2;
    return ret;
  }

  static __inline CvMyCorrespond cvMyCorrespond3(CvPoint2D32f pt1,
						 CvPoint2D32f pt2,
						 CvPoint2D32f pt3){
    CvMyCorrespond ret;
    ret.num = 3;
    ret.point[0] = pt1;
    ret.point[1] = pt2;
    ret.point[2] = pt3;
    return ret;
  }

  int searchCorrespond2(IplImage *img1,CvPoint2D32f *point1,int pnum1,
			IplImage *img2,CvPoint2D32f *point2,int pnum2,
			CvMyCorrespond **correspond_ret,int *cor_num_ret,
			CvMat *F,
			double cor_min,double lenght);
  // 2枚の画像間の組み合わせを計算する．
  // img1,img2 : 入力画像
  // point1,point2 : 特徴点，数が同じ必用はない
  // pnum1,pnum2 : 各特徴点の数
  // correspond_ret : 対応点組を格納，NULLであれば何も返さない
  // cor_num_ret : 対応点組の数
  // F : 計算過程で得られるF行列を格納
  // cor_min : 対応点相関の最小値
  // length : エピポーラ線との距離の閾値
  
  int searchCorrespond3(IplImage *img1,CvPoint2D32f *point1,int pnum1,
			IplImage *img2,CvPoint2D32f *point2,int pnum2,
			IplImage *img3,CvPoint2D32f *point3,int pnum3,
			CvMat *F12,CvMat *F13,CvMat *F23,
			CvMyCorrespond **correspond_ret,
			int *cor_num_ret,
			double cor_min,double lenght);
  // 3枚の画像間の対応点を計算する
  // F行列を利用して正確さを向上させる
  // Fxy : x -> y の F 行列，NULLの場合は使用されない
  // それ以外は searchCorrespond2と同様
  


#ifdef __cplusplus
}
#endif

#endif
