// 3次元復元

#include <cv.h>

#include "linear.h"
#include "recover.h"

int cvReconst3DFrom3Camera(const CvMat *P1,
			   const CvMat *P2,
			   const CvMat *P3,
			   const CvMat *point1,
			   const CvMat *point2,
			   const CvMat *point3,
			   CvMat *reconst){
  // 3カメラから復元
  CvMat *M;
  const CvMat *Pset[3];
  const CvMat *pointset[3];
  int i,j;
  
  Pset[0] = P1;
  Pset[1] = P2;
  Pset[2] = P3;
  
  pointset[0] = point1;
  pointset[1] = point2;
  pointset[2] = point3;

  M = cvCreateMat(6,4,CV_64F);
  
  for (i=0;i<3;i++){
    for (j=0;j<4;j++){
      cvmSet(M,i*2,j,
	     cvGetReal1D(pointset[i],0) * cvmGet(Pset[i],2,j)
	     - cvmGet(Pset[i],0,j));
      cvmSet(M,i*2+1,j,
	     cvGetReal1D(pointset[i],1) * cvmGet(Pset[i],2,j)
	     - cvmGet(Pset[i],1,j));
    }
  }

  cvGetMinEigenVector(M,reconst);
  
  // 正規化
  for (i=0;i<3;i++)
    cvSetReal1D(reconst,i,
		cvGetReal1D(reconst,i) / cvGetReal1D(reconst,3));
  cvSetReal1D(reconst,3,(double)1);
  
  cvReleaseMat(&M);

  return 0;
}
