// テクスチャマッピングを行う
// OpenCV を使うより OpenGL 等を利用する方が無難

#include <cv.h>

#include "texture.h"

/* ----- prototype ----- */
static CvRect myBoundingRect32f(int num,CvPoint2D32f *point);
/* ----- prototype ----- */

int drawTriangleTexture(IplImage *src, CvPoint2D32f *psrc,
			IplImage *dst, CvPoint2D32f *pdst){
  // src -> dst へのアフィン変換を含むコピー
  // いわゆるテクスチャマッピング
  // src : コピー元画像
  // psrc : コピー元の画像点（3各パッチの頂点）
  // dst : コピー先画像
  // pdst : コピー先の画像点（3各パッチの頂点）

  CvMat *affine;
  IplImage *temp,*mask;
  int vertex[1]={3};
  CvPoint *pdst2;
  CvPoint2D32f ps[3],pd[3];
  CvRect rect1,rect2;
  int i;
  
  // 対象となる矩形を決定
  rect1 = myBoundingRect32f(3,psrc);
  rect2 = myBoundingRect32f(3,pdst);

  cvSetImageROI(src,rect1);
  cvSetImageROI(dst,rect2);
  
  // 矩形領域内での座標を計算(最初の3点のみを使用)
  for (i=0;i<3;i++){
    ps[i].x = psrc[i].x - rect1.x;
    ps[i].y = psrc[i].y - rect1.y;
    pd[i].x = pdst[i].x - rect2.x;
    pd[i].y = pdst[i].y - rect2.y;
  }

  // 矩形 -> 矩形へのアフィン変換を計算
  affine = cvCreateMat(2,3,CV_64F);
  affine = cvGetAffineTransform(ps,pd,affine);
  
  // アフィン変換により画像生成(生成画像をなるべく小さくする)
  temp = cvCreateImage(cvSize(rect2.width,rect2.height),
		       dst->depth,dst->nChannels);
  cvWarpAffine(src,temp,affine,CV_INTER_LINEAR+CV_WARP_FILL_OUTLIERS,
	       cvScalarAll(0));
  
  // mask 作成
  mask = cvCreateImage(cvGetSize(temp),IPL_DEPTH_8U,1);
  cvSetZero(mask);

  // ポリゴンの内側を塗りつぶした画像をマスクとして使用
  pdst2 = (CvPoint*)calloc(3,sizeof(CvPoint));
  for (i=0;i<3;i++){
    pdst2[i].x = pd[i].x;
    pdst2[i].y = pd[i].y;
  }
  cvFillPoly(mask,&pdst2,vertex,1,cvScalar(255,0,0,0));

  // mask つきで copy
  cvCopy(temp,dst,mask);
  cvPolyLine(dst,&pdst2,vertex,1,1, cvScalar(0,0,255));

  // ROI を reset
  cvResetImageROI(src);
  cvResetImageROI(dst);

  // 後始末
  cvReleaseImage(&temp);
  cvReleaseImage(&mask);
  free(pdst2);

  return 0;
}

int drawTriangleTexture2(IplImage *src,
			 CvPoint2D32f psrc1,
			 CvPoint2D32f psrc2,
			 CvPoint2D32f psrc3,
			 IplImage *dst,
			 CvPoint2D32f pdst1,
			 CvPoint2D32f pdst2,
			 CvPoint2D32f pdst3){
  // 上と同じくテクスチャマッピング
  // 配列を用意せずに使用可能
  CvPoint2D32f psrc[3],pdst[3];

  psrc[0] = psrc1;
  psrc[1] = psrc2;
  psrc[2] = psrc3;
  
  pdst[0] = pdst1;
  pdst[1] = pdst2;
  pdst[2] = pdst3;

  return drawTriangleTexture(src,psrc,dst,pdst);
}

static CvRect myBoundingRect32f(int num,CvPoint2D32f *point){
  int i;
  double minx=(double)0xffff,miny=(double)0xffff,
    maxx=0,maxy=0;

  for (i=0;i<num;i++){
    if (point[i].x>maxx) maxx = point[i].x;
    if (point[i].y>maxy) maxy = point[i].y;
    if (point[i].x<minx) minx = point[i].x;
    if (point[i].y<miny) miny = point[i].y;
  }
  return cvRect((int)minx,(int)miny,
		(int)(maxx-minx),(int)(maxy-miny));
}

