#ifndef TRIFOCAL_TENSOR_H
#define TRIFOCAL_TENSOR_H

#include "correspond.h"

#ifdef __cplusplus
extern "C"{
#endif

  int findTFTensorRansac(CvMyCorrespond *correspond, int num,
			 CvMat *inlier_ret,
			 CvMatND *tftensor,
			 int loopnum,double lenght);
  // 対応点組から RANSAC を利用して trifocal tensor を計算
  // correspond : 対応点組
  // num : 対応点組の数
  // inlier_ret : RANSACにより得られる inlier のフラグを格納
  // tftensor : 戻り値となる trifocal tensor
  // loopnum : ransac のループ回数を指定，多いとロバスト性が向上
  // length : 点変換後の点との距離の閾値
  
  CvMatND *cvFindTrifocalTensorLinear2(const CvMat *point1,
				       const CvMat *point2,
				       const CvMat *point3);

  int transPointByTFTensor2D32f(const CvMatND *tensor,
				const CvPoint2D32f pt1,
				const CvPoint2D32f pt2,
				CvPoint2D32f *pt3);
  
  int cvTFTensorPointTransfer(const CvMatND *tensor,
			      const CvMat *point1,
			      const CvMat *point2,
			      CvMat *point3);

  int cvTFTensorFindEpipoles(const CvMatND *tensor,
			     CvMat *epi1,
			     CvMat *epi2);

  int cvTFTensorFindCameraMat(const CvMatND *tensor,
			      CvMat *cam1,
			      CvMat *cam2);

  int cvFindTFTensorNormalize(const CvMat *point1,
			      const CvMat *point2,
			      const CvMat *point3,
			      CvMatND *tensor);

  int cvFindTFTensorRANSAC(const CvMat *point1,
			   const CvMat *point2,
			   const CvMat *point3,
			   int num,
			   int *inliers,
			   CvMatND *tensor,
			   int maxloop,double thre);
  
#ifdef __cplusplus
}
#endif

#endif
